-- /////////////////////////////////////////////////////////////////////////////
-- =============================================================================
--  CLEARFONT2 BY KIRKBURN
--  Official website:  http://www.clearfont.co.uk/
-- -----------------------------------------------------------------------------
--  DESIGN.LUA - THE CLEARFONT DESIGN CODE
--	A. SETTING UP THE ADDON
--	B. WOW UI DESIGN
--  	C. DYNAMIC ADDON UPDATES
--  	D. AND FINALLY...
-- =============================================================================
-- /////////////////////////////////////////////////////////////////////////////



-- =============================================================================
--  A. SETTING UP THE ADDON
-- =============================================================================

-- -----------------------------------------------------------------------------
-- Funky local fun! (ckknight)
-- -----------------------------------------------------------------------------

local tmp = {}
local function iter(t, i)
	i = i + 1
	local x = t[i]
	if not x then
		return
	end
	return i, x
end
local function fake_ipairs(...)
	for k in pairs(tmp) do
		tmp[k] = nil
	end
	local n = select('#', ...)
	for i = 1, n do
		tmp[i] = select(i, ...)
	end
	return iter, tmp, 0
end

local last_scale = nil
local last_font1 = nil
local last_font2 = nil
local last_font3 = nil
local last_font4 = nil
local last_font5 = nil


-- -----------------------------------------------------------------------------
-- The function called by Core.lua
-- -----------------------------------------------------------------------------

function ApplyClearFontDesign()


-- =============================================================================
--  B. WOW UI DESIGN
-- =============================================================================
--  This is the most important section to edit!
--  Main font styles are listed first, the rest follows alphabetically
-- =============================================================================

	local scale					= ClearFont.db.profile.currentScale
	local NORMAL_TEXT_FONT		= ClearFont.fonts[index].normal
	local BOLD_TEXT_FONT		= ClearFont.fonts[index].bold
	local BOLDITALIC_TEXT_FONT	= ClearFont.fonts[index].bolditalic
	local ITALIC_TEXT_FONT		= ClearFont.fonts[index].italic
	local NUMBER_TEXT_FONT		= ClearFont.fonts[index].number
	local scalechange 			= last_scale ~= scale
	last_scale 					= scale

-- -----------------------------------------------------------------------------
-- In-world, '3D' fonts, etc. (Dark Imakuni)
-- -----------------------------------------------------------------------------

-- Names above people's heads
	UNIT_NAME_FONT = NORMAL_TEXT_FONT
	NAMEPLATE_FONT = NORMAL_TEXT_FONT

-- The damage pop-up over your target's head (*not* SCT/SDT)
	DAMAGE_TEXT_FONT = NUMBER_TEXT_FONT

-- Chat bubbles
	STANDARD_TEXT_FONT = NORMAL_TEXT_FONT

-- Drop-down menu font
	UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT = 12 * scale


-- -----------------------------------------------------------------------------
-- System Font
-- -----------------------------------------------------------------------------

SystemFont:SetFont(NORMAL_TEXT_FONT, 17 * scale)


-- -----------------------------------------------------------------------------
-- Primary Game Fonts: the main font used, e.g. window titles
-- -----------------------------------------------------------------------------

GameFontNormal:SetFont(NORMAL_TEXT_FONT, 13 * scale)
GameFontHighlight:SetFont(NORMAL_TEXT_FONT, 13 * scale)

GameFontDisable:SetFont(NORMAL_TEXT_FONT, 13 * scale)
GameFontDisable:SetTextColor(0.6, 0.6, 0.6)

GameFontGreen:SetFont(NORMAL_TEXT_FONT, 13 * scale)
GameFontRed:SetFont(NORMAL_TEXT_FONT, 13 * scale)
GameFontBlack:SetFont(NORMAL_TEXT_FONT, 13 * scale)
GameFontWhite:SetFont(NORMAL_TEXT_FONT, 13 * scale)


-- -----------------------------------------------------------------------------
-- Small Game Fonts: the main small font, e.g. character sheets, buff timers, Titan Panel, etc
-- -----------------------------------------------------------------------------

GameFontNormalSmall:SetFont(BOLD_TEXT_FONT, 11 * scale)

GameFontHighlightSmall:SetFont(BOLD_TEXT_FONT, 11 * scale)
GameFontHighlightSmallOutline:SetFont(BOLD_TEXT_FONT, 11 * scale, "OUTLINE")

GameFontDisableSmall:SetFont(BOLD_TEXT_FONT, 11 * scale)
GameFontDisableSmall:SetTextColor(0.6, 0.6, 0.6)

GameFontDarkGraySmall:SetFont(BOLD_TEXT_FONT, 11 * scale)
GameFontDarkGraySmall:SetTextColor(0.4, 0.4, 0.4)

GameFontGreenSmall:SetFont(BOLD_TEXT_FONT, 11 * scale)
GameFontRedSmall:SetFont(BOLD_TEXT_FONT, 11 * scale)


-- -----------------------------------------------------------------------------
-- Large Game Fonts: titles
-- -----------------------------------------------------------------------------

GameFontNormalLarge:SetFont(BOLD_TEXT_FONT, 17 * scale)
GameFontHighlightLarge:SetFont(NORMAL_TEXT_FONT, 17 * scale)

GameFontDisableLarge:SetFont(NORMAL_TEXT_FONT, 17 * scale)
GameFontDisableLarge:SetTextColor(0.6, 0.6, 0.6)

GameFontGreenLarge:SetFont(NORMAL_TEXT_FONT, 17 * scale)
GameFontRedLarge:SetFont(NORMAL_TEXT_FONT, 17 * scale)


-- -----------------------------------------------------------------------------
-- Huge Game Fonts: raid warnings
-- -----------------------------------------------------------------------------

GameFontNormalHuge:SetFont(BOLD_TEXT_FONT, 20 * scale)


-- -----------------------------------------------------------------------------
-- CombatTextFont: in-built SCT-style info
-- -----------------------------------------------------------------------------

CombatTextFont:SetFont(NORMAL_TEXT_FONT, 26 * scale)


-- -----------------------------------------------------------------------------
-- Number Fonts: e.g. Auction House, money, keybindings, & quantity overlays
-- -----------------------------------------------------------------------------

NumberFontNormal:SetFont(NUMBER_TEXT_FONT, 15 * scale, "OUTLINE")
NumberFontNormalYellow:SetFont(NUMBER_TEXT_FONT, 15 * scale, "OUTLINE")
NumberFontNormalSmall:SetFont(NUMBER_TEXT_FONT, 13 * scale, "OUTLINE")
NumberFontNormalSmallGray:SetFont(NUMBER_TEXT_FONT, 13 * scale, "OUTLINE")
NumberFontNormalLarge:SetFont(NUMBER_TEXT_FONT, 17 * scale, "OUTLINE")

NumberFontNormalHuge:SetFont(NUMBER_TEXT_FONT, 30 * scale, "THICKOUTLINE")
NumberFontNormalHuge:SetAlpha(30)


-- -----------------------------------------------------------------------------
-- ChatFrame: change chat font, set edit box size, and add more font size options
-- -----------------------------------------------------------------------------

ChatFontNormal:SetFont(NORMAL_TEXT_FONT, 14 * scale)

-- This would be used to set individual chat frames if it didn't just break everything :)
-- ChatFrame1:SetFont(NORMAL_TEXT_FONT, 14 * scale)
-- ChatFrame2:SetFont(NORMAL_TEXT_FONT, 14 * scale)
-- ChatFrame3:SetFont(NORMAL_TEXT_FONT, 14 * scale)
-- ChatFrame4:SetFont(NORMAL_TEXT_FONT, 14 * scale)
-- ChatFrame5:SetFont(NORMAL_TEXT_FONT, 14 * scale)
-- ChatFrame6:SetFont(NORMAL_TEXT_FONT, 14 * scale)
-- ChatFrame7:SetFont(NORMAL_TEXT_FONT, 14 * scale)

CHAT_FONT_HEIGHTS = {
	[1] = 7,
	[2] = 8,
	[3] = 9,
	[4] = 10,
	[5] = 11,
	[6] = 12,
	[7] = 13,
	[8] = 14,
	[9] = 15,
	[10] = 16,
	[11] = 17,
	[12] = 18,
	[13] = 19,
	[14] = 20,
	[15] = 21,
	[16] = 22,
	[17] = 23,
	[18] = 24
}


-- -----------------------------------------------------------------------------
-- Quest Log: e.g. quest log, books, etc.
-- -----------------------------------------------------------------------------

QuestTitleFont:SetFont(BOLD_TEXT_FONT, 19 * scale)
QuestTitleFont:SetShadowColor(0.54, 0.4, 0.1)

QuestFont:SetFont(ITALIC_TEXT_FONT, 14 * scale)
QuestFont:SetTextColor(0.15, 0.09, 0.04)

QuestFontNormalSmall:SetFont(BOLD_TEXT_FONT, 13 * scale)
QuestFontNormalSmall:SetShadowColor(0.54, 0.4, 0.1)

QuestFontHighlight:SetFont(NORMAL_TEXT_FONT, 15 * scale)


-- -----------------------------------------------------------------------------
-- Dialog Buttons: e.g. "Accept", etc.
-- -----------------------------------------------------------------------------

DialogButtonNormalText:SetFont(NORMAL_TEXT_FONT, 17 * scale)
DialogButtonHighlightText:SetFont(NORMAL_TEXT_FONT, 17 * scale)


-- -----------------------------------------------------------------------------
-- Error Log: e.g. "Another Action is in Progress", etc.
-- -----------------------------------------------------------------------------

ErrorFont:SetFont(ITALIC_TEXT_FONT, 16 * scale)
ErrorFont:SetAlpha(60)


-- -----------------------------------------------------------------------------
-- Item Info: general usage in frames
-- -----------------------------------------------------------------------------

ItemTextFontNormal:SetFont(NORMAL_TEXT_FONT, 16 * scale)


-- -----------------------------------------------------------------------------
-- Mail & Invoice Text: in-game mails & Auction House invoices
-- -----------------------------------------------------------------------------

MailTextFontNormal:SetFont(ITALIC_TEXT_FONT, 15 * scale)
MailTextFontNormal:SetTextColor(0.15, 0.09, 0.04)
MailTextFontNormal:SetShadowColor(0.54, 0.4, 0.1)
MailTextFontNormal:SetShadowOffset(1, -1)

InvoiceTextFontNormal:SetFont(ITALIC_TEXT_FONT, 13 * scale)
InvoiceTextFontNormal:SetTextColor(0.15, 0.09, 0.04)

InvoiceTextFontSmall:SetFont(ITALIC_TEXT_FONT, 11 * scale)
InvoiceTextFontSmall:SetTextColor(0.15, 0.09, 0.04)


-- -----------------------------------------------------------------------------
-- PvP Objectives Info
-- -----------------------------------------------------------------------------

PVPInfoTextFont:SetFont(NORMAL_TEXT_FONT, 22 * scale, "THICKOUTLINE")


-- -----------------------------------------------------------------------------
-- Spell Book: ability subtitles
-- -----------------------------------------------------------------------------

SubSpellFont:SetFont(BOLD_TEXT_FONT, 11 * scale)


-- -----------------------------------------------------------------------------
-- Status Bars: numbers on the unit frames, Damage Meters
-- -----------------------------------------------------------------------------

TextStatusBarText:SetFont(NUMBER_TEXT_FONT, 12 * scale, "OUTLINE")
TextStatusBarTextSmall:SetFont(NORMAL_TEXT_FONT, 13 * scale)


-- -----------------------------------------------------------------------------
-- Tooltips: text used in tooltips
-- -----------------------------------------------------------------------------

GameTooltipText:SetFont(NORMAL_TEXT_FONT, 13 * scale)
GameTooltipTextSmall:SetFont(BOLD_TEXT_FONT, 12 * scale)
GameTooltipHeaderText:SetFont(BOLD_TEXT_FONT, 15 * scale, "OUTLINE")


-- -----------------------------------------------------------------------------
-- World Map: location titles
-- -----------------------------------------------------------------------------

WorldMapTextFont:SetFont(BOLDITALIC_TEXT_FONT, 31 * scale, "THICKOUTLINE")
WorldMapTextFont:SetShadowColor(0, 0, 0)
WorldMapTextFont:SetShadowOffset(1, -1)
WorldMapTextFont:SetAlpha(40)


-- -----------------------------------------------------------------------------
-- Zone Text: on-screen notifications of zone changes
-- -----------------------------------------------------------------------------

ZoneTextFont:SetFont(BOLDITALIC_TEXT_FONT, 31 * scale, "THICKOUTLINE")
ZoneTextFont:SetShadowColor(0, 0, 0)
ZoneTextFont:SetShadowOffset(1, -1)

SubZoneTextFont:SetFont(BOLDITALIC_TEXT_FONT, 27 * scale, "THICKOUTLINE")


-- -----------------------------------------------------------------------------
-- Unknown: may no longer be used
-- -----------------------------------------------------------------------------

CombatLogFont:SetFont(NORMAL_TEXT_FONT, 13 * scale)



-- =============================================================================
--  C. DYNAMIC ADDON UPDATES
-- =============================================================================
--  Code. It does stuff, supposedly. (ckknight)
-- =============================================================================

if not scalechange and ClearFont.db.profile.applyAll then
	local frame
	local last_font1 = last_font1
	local last_font2 = last_font2
	local last_font3 = last_font3
	local last_font4 = last_font4
	local last_font5 = last_font5
	ClearFont:ScheduleRepeatingEvent("ClearFont2-DynamicUpdate", function()
		local t = GetTime() + 0.05
		while true do
			if GetTime() > t then
				return
			end
			frame = EnumerateFrames(frame)
			if not frame then
				break
			end
			for _, region in fake_ipairs(frame:GetRegions()) do
				if region:IsObjectType("FontString") then
					local obj = region:GetFontObject()
					if obj then
						local font,size,style = region:GetFont()
						if font == last_font1 or font == last_font2 or font == last_font3 or font == last_font4 or font == last_font5 then
							font = obj:GetFont()
							if font then
								region:SetFont(font,size,style)
							end
						end
					end
				end
			end
		end
		ClearFont:CancelScheduledEvent("ClearFont2-DynamicUpdate")
	end, 0)
end

last_font1 = NORMAL_TEXT_FONT
last_font2 = BOLD_TEXT_FONT
last_font3 = BOLDITALIC_TEXT_FONT
last_font4 = ITALIC_TEXT_FONT
last_font5 = NUMBER_TEXT_FONT



-- =============================================================================
--  D. AND FINALLY...
-- =============================================================================

end
